import requests
from bs4 import BeautifulSoup
import pandas as pd
import numpy as np
import os


def extract_tickers_and_capitalization(url="https://smart-lab.ru/q/shares_fundamental/?field=market_cap"):
    """
    Extracts tickers and their capitalization from a given URL.

    Args:
        url: The URL of the webpage containing the table.

    Returns:
        A dictionary where keys are tickers and values are their capitalization, 
        or None if there's an error.
    """
    try:
        response = requests.get(url)
        response.raise_for_status()  # Raise HTTPError for bad responses (4xx or 5xx)
        soup = BeautifulSoup(response.content, 'html.parser')

        # Find the table (adjust selector if needed based on website structure)
        table = soup.find('table') #or a more specific selector if needed (e.g., soup.find('table', {'id': 'my-table'}))
        if table is None:
            return None  #Table not found

        tickers_cap = {}
        for row in table.find_all('tr'):
            cols = row.find_all('td')
            if len(cols) >= 5:  # Ensure there are enough columns
                ticker = cols[2].text.strip()
                capitalization = cols[5].text.strip()
                #print(ticker, capitalization)
                try:
                    capitalization = float(capitalization.replace(' ', ''))  # Clean and convert to int
                    tickers_cap[ticker] = capitalization
                except ValueError:
                    print(f"Could not parse capitalization for {ticker}.")

        return tickers_cap

    except requests.exceptions.RequestException as e:
        print(f"An error occurred during the request: {e}")
        return None
    except Exception as e:
        print(f"An unexpected error occurred: {e}")
        return None
    

def extract_multipliers(ticker):

    def download_mult(ticker):
        link = f'https://smart-lab.ru/q/{ticker}/f/q/MSFO/download/'
        response = requests.get(link, stream=True, verify=False)  # stream=True is good for large files
        response.raise_for_status() # Raise an exception for bad status codes (4xx or 5xx)
        filename = f'{ticker}_data.csv'  #or get filename from headers if available

        with open(filename, 'wb') as f:  # 'wb' for writing binary data
            for chunk in response.iter_content(chunk_size=8192): #Iterate chunk-wise for memory efficiency
                f.write(chunk)
        df = pd.read_csv(filename, sep=';')  # read the downloaded file
        os.remove(filename)
        df = df.set_index('Unnamed: 0').T
        return df
    
    def correct_date(df):
        def func(x):
            if x == 'LTM':
                return np.nan
            year = x[:4]
            quarter = int(x[5:])
            dict_ = {1:'01-01', 2: '04-01-', 3: '07-01', 4: '10-01'}
            date = f'{year}-{dict_[quarter]}'
            return pd.to_datetime(date)
        
        data = df.reset_index()
        data['dt'] = data['index'].apply(func)
        del data['index']
        data = data[['dt'] + list(data.columns[:-1])]
        return data